@@ -34,7 +34,8 @@
 #   under Cygwin the Windows version is used
 #
 ltdirectory = '../LT/LanguageTool-4.7/'
-ltcommand = 'java -jar ' +  ltdirectory + 'languagetool-commandline.jar'
+ltcommand = lambda: ('java -jar ' +  cmdline.lt_directory
+                                + 'languagetool-commandline.jar')
 
 # on option --server lt: address of server hosted by LT
 #
@@ -53,6 +54,7 @@ config_file = '.t2t-shell'
 
 # default option values
 #
+default_option_lt_directory = ltdirectory
 default_option_language = 'en-GB'
 default_option_encoding = 'utf-8'
 default_option_disable = 'WHITESPACE_RULE'
@@ -121,16 +123,19 @@ import urllib.parse
 import urllib.request
 import time
 import signal
+import xml.etree.ElementTree as ET
 
 # parse command line
 #
 parser = argparse.ArgumentParser()
-parser.add_argument('--html', action='store_true')
+parser.add_argument('--lt-directory', default=default_option_lt_directory)
+parser.add_argument('--output', choices=['plain', 'html', 'xml'],
+                                                    default='plain')
 parser.add_argument('--link', action='store_true')
 parser.add_argument('--context', type=int)
 parser.add_argument('--include', action='store_true')
 parser.add_argument('--skip')
-parser.add_argument('--plain', action='store_true')
+parser.add_argument('--plain-input', action='store_true')
 parser.add_argument('--list-unknown', action='store_true')
 parser.add_argument('--language')
 parser.add_argument('--t2t-lang')
@@ -158,6 +163,8 @@ except:
     config = []
 cmdline = parser.parse_args(config + sys.argv[1:])
 
+if not cmdline.lt_directory.endswith('/'):
+    cmdline.lt_directory += '/'
 if cmdline.language is None:
     cmdline.language = default_option_language
 if cmdline.t2t_lang is None:
@@ -173,8 +180,9 @@ if cmdline.context < 0:
     cmdline.context = int(1e8)
 if cmdline.server is not None and cmdline.server not in ('lt', 'my', 'stop'):
     tex2txt.fatal('mode for --server has to be one of lt, my, stop')
-if cmdline.plain and (cmdline.include or cmdline.replace):
-    tex2txt.fatal('cannot handle --plain together with --include or --replace')
+if cmdline.plain_input and (cmdline.include or cmdline.replace):
+    tex2txt.fatal('cannot handle --plain-input together with'
+                                        + ' --include or --replace')
 if cmdline.single_letters and cmdline.single_letters.endswith('||'):
     cmdline.single_letters += equation_replacements
 if cmdline.replace:
@@ -208,7 +216,7 @@ if cmdline.server == 'stop':
 
 # complement LT options
 #
-ltcommand = ltcommand.split() + ['--json', '--encoding', 'utf-8',
+ltcommand = ltcommand().split() + ['--json', '--encoding', 'utf-8',
                             '--language', cmdline.language]
 if cmdline.disable:
     ltcommand += ['--disable', cmdline.disable]
@@ -294,7 +302,7 @@ def run_proofreader(file):
     if not tex.endswith('\n'):
         tex += '\n'
 
-    if cmdline.plain:
+    if cmdline.plain_input:
         (plain, charmap) = (tex, list(range(1, len(tex) + 1)))
     else:
         (plain, charmap) = tex2txt.tex2txt(tex, options)
@@ -412,14 +420,14 @@ def start_local_lt_server():
     if check_server():
         return
     try:
-        subprocess.Popen(ltserver_local_cmd.split(), cwd=ltdirectory,
+        subprocess.Popen(ltserver_local_cmd.split(), cwd=cmdline.lt_directory,
                         stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
     except:
         tex2txt.fatal('error running "' + ltserver_local_cmd + '"')
 
     # wait for server to be available
     #
-    sys.stderr.write('=== starting local LT server at "' + ltdirectory
+    sys.stderr.write('=== starting local LT server at "' + cmdline.lt_directory
                         + '":\n=== ' + ltserver_local_cmd + ' ')
     sys.stderr.flush()
     for x in range(20):
@@ -574,7 +582,7 @@ def create_message(m, rule, msg, repl):
         'offset': offset,
         'length': length,
         'context': create_context(m.string, offset, length),
-        'rule': {'id': rule},
+        'rule': {'id': rule, 'category': {'name': 'Problem'}},
         'message': msg,
         'replacements': [{'value': repl}],
     }
@@ -642,7 +650,7 @@ def output_list_unknown(unkn, file):
     print(unkn)
 
 
-if not cmdline.html or cmdline.list_unknown:
+if cmdline.output == 'plain' or cmdline.list_unknown:
     if cmdline.server == 'lt':
         sys.stderr.write(msg_LT_server_txt)
     for file in cmdline.file:
@@ -654,6 +662,71 @@ if not cmdline.html or cmdline.list_unknown:
     sys.exit()
 
 
+#####################################################################
+#
+#   XML report for vim-grammarous
+#
+#####################################################################
+
+#   - XXX: some code duplication with begin_match()
+#
+def output_xml_report(tex, plain, charmap, matches, file, out):
+    starts = tex2txt.get_line_starts(plain)
+    out.write('<matches>\n')
+    for m in matches:
+        offset = json_get(m, 'offset', int)
+        lin = plain.count('\n', 0, offset) + 1
+        nl = plain.rfind('\n', 0, offset) + 1
+        col = offset - nl + 1
+        lc = tex2txt.translate_numbers(tex, plain, charmap, starts, lin, col)
+        fromy = lc.lin - 1
+        fromx = lc.col - 1
+
+        length = json_get(m, 'length', int)
+        end = offset + length - 1
+        lin = plain.count('\n', 0, end) + 1
+        nl = plain.rfind('\n', 0, end) + 1
+        col = end - nl + 1
+        lc = tex2txt.translate_numbers(tex, plain, charmap, starts, lin, col)
+        toy = lc.lin - 1
+        tox = lc.col
+
+        rule = json_get(m, 'rule', dict)
+        category = json_get(rule, 'category', dict)
+        category = json_get(category, 'name', str)
+        message = json_get(m, 'message', str)
+        repls = '#'.join(json_get(r, 'value', str)
+                                for r in json_get(m, 'replacements', list))
+        cont = json_get(m, 'context', dict)
+        cont_text = json_get(cont, 'text', str)
+        cont_offset = json_get(cont, 'offset', int)
+        cont_length = json_get(cont, 'length', int)
+
+        xml = {
+            'fromy': str(fromy), 'fromx': str(fromx),
+            'toy': str(toy), 'tox': str(tox),
+            'category': category,
+            'msg': message,
+            'replacements': repls,
+            'context': cont_text,
+            'contextoffset': str(cont_offset),
+            'errorlength': str(cont_length),
+        }
+        s = ET.tostring(ET.Element('error', xml), encoding='unicode') + '\n'
+        out.write(s)
+    out.write('</matches>\n')
+
+
+if cmdline.output == 'xml':
+    if cmdline.server == 'lt':
+        sys.stderr.write(msg_LT_server_txt)
+    out = open(sys.stdout.fileno(), mode='w', encoding='utf-8')
+    for file in cmdline.file:
+        (tex, plain, charmap, matches) = run_proofreader(file)
+        output_xml_report(tex, plain, charmap, matches, file, out)
+    sys.exit()
+
+
 #####################################################################
 #
 #   HTML report
